#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.2";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","d=s","b=s","g=s","o=s","c=s","h" );

#&help()if(defined $opts{h});
if((!defined($opts{i}) && !defined $opts{d})|| !defined($opts{g}) || !defined($opts{b}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           tags bamfile                 <infile>     optional with -d
		-d           tags bamfiles dir            <indir>      optioanl with -i
		-b           barcode seq file             <infile>     must be given
		-g           gtf/gff file                 <infile>     must be given
		-o           result file                  <outfile>    must be given
		-c           barcode.tsv file             [optional]   optional     
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $outfile = $opts{o} ;
my $bcfile = $opts{b} ;
my $gtffile = $opts{g} ;
my $bctsvfile = defined $opts{c} ? $opts{c} : "" ;

## read barcode file
my %hbc = ();
&get_barcode($bcfile, \%hbc);

## get gene info
my %hgene = ();
my %hgeneori = ();
&get_gene_info($gtffile, \%hgene, \%hgeneori);

## 
my %hstat = ();
my %hbc_stat = ();
my %hsingle_stat = ();
my $total_reads = 0 ;
my %hgene_map = ();
if (defined $opts{i}){
    my $infile = $opts{i} ;
    &show_log("process $infile");
    $total_reads = &get_tags_info($infile, $outfile, \%hbc, \%hgene, \%hgeneori, \%hstat, \%hbc_stat, \%hsingle_stat, \%hgene_map);
}
elsif (defined $opts{d}){
    my $indir = $opts{d} ;
    my @files = glob("$indir/*.bam") ;
    for my $infile (@files){
        &show_log("process $infile");
        $total_reads += &get_tags_info($infile, $outfile, \%hbc, \%hgene, \%hgeneori, \%hstat, \%hbc_stat, \%hsingle_stat, \%hgene_map);
    }
}

open (STAT, ">$outfile.map_stat") || die "$outfile.map_stat, $!\n" ;
print STAT "TotalMapped\t$total_reads\n" ;
for my $key (sort keys %hgene_map){
    my $value = $hgene_map{$key} ;
    print STAT "$key\t$value\n" ;
}
close(STAT);

open (STAT, ">$outfile.bc_stat") || die "$outfile.bc_stat, $!\n" ;
print STAT "Total_reads\t", $total_reads, "\n" ;
for my $bc (sort keys %hbc_stat){
    my $num = $hbc_stat{$bc} ;
    my $perc = int($num/$total_reads*10000+0.5)/100 ;
    print STAT "$bc\t$num\t$perc\n" ;
}
print STAT "=========== single stat ============\n" ;
for my $bc (sort keys %hsingle_stat){
    my $num = $hsingle_stat{$bc} ;
    my $perc = int($num/$total_reads*10000+0.5)/100 ;
    print STAT "$bc\t$num\t$perc\n" ;
}
close(STAT) ;

## out put
&output(\%hstat, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_barcode($bcfile, \%hbc);
sub get_barcode()
{
    my ($bcfile, $ahbc) = @_ ;

    open (IN, $bcfile) || die "$bcfile, $!\n" ;
    $/ = ">" ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$/);
        my ($id, $seq) = split(/\n/, $_, 2) ;
        $id = (split /\s+/, $id)[0] ;
        $seq =~ s/\n//g ;
        $ahbc->{$seq} = $id ;
    }
    $/ = "\n" ;
    close(IN);

    return ;
}

#&get_gene_info($gtffile, \%hgene, \%hgeneori);
sub get_gene_info()
{
    my ($gtffile, $ahgene, $ahgeneori) = @_ ;

    my %htrans = ();
    if ($gtffile =~ /\.gtf$/){
        open (IN, $gtffile) || die "$gtffile, $!\n" ;
        while(<IN>){
            chomp ;
            next if (m/^\#|^\s*$/);
            my ($chr, $soft, $type, $start, $end, undef, $ori, $offset, $info) = split(/\t+/, $_, 9) ;
            next if ($type ne 'exon') ;
            my @infos = split /\s+/, $info ;
            my %hinfos = ();
            for (my $i=0; $i<@infos; $i+=2){
                $hinfos{$infos[$i]} = $infos[$i+1] ;
            }
            my $gene_id = $hinfos{"gene_id"} ;
            $gene_id =~ s/[\"\;]//g ;
            push @{$htrans{$gene_id}}, [$start, $end] ;
            $ahgeneori->{$gene_id} = $ori ;
        }
        close(IN);
    }
    elsif ($gtffile =~ /\.gff$|\.gff3$/){
        open (IN, $gtffile) || die "$gtffile, $!\n" ;
        my $gene_id = "" ;
        while(<IN>){
            chomp ;
            next if (m/^\#|^\s*$/);
            my ($chr, $soft, $type, $start, $end, undef, $ori, $offset, $info) = split(/\t+/, $_, 9) ;
            if ($type =~ m/gene/i){
                next if ($type =~ /segment/) ;
                if ($info =~ m/ID=(.*?)\;/){
                    $gene_id = $1 ;
                }
                elsif ($info =~ m/ID=([^;]*?)\s*$/){
                    $gene_id = $1 ;
                }
                $gene_id =~ s/^gene[\:\-\_]// ;
                $ahgeneori->{$gene_id} = $ori ;
            }
            elsif ($type eq 'exon'){
                push @{$htrans{$gene_id}}, [$start, $end] ;
            }
        }
        close(IN);
    }
    else{
    }

    for my $gene (keys %htrans){
        my @region = sort {$a->[0]<=>$b->[0] || $a->[1]<=>$b->[1]} @{$htrans{$gene}} ;
        my @merge = ($region[0]) ;
        for (my $i=1; $i<@region; $i++){
            if ($region[$i][0] < $merge[-1][1]){
                $merge[-1][1] = $region[$i][1] > $merge[-1][1] ? $region[$i][1] : $merge[-1][1] ;
            }
            else{
                push @merge, $region[$i] ;
            }
        }
        $ahgene->{$gene} = \@merge ;
    }

    return ;
}

#$total_reads = &get_tags_info($infile, $outfile, \%hbc, \%hgene, \%hgeneori, \%hstat, \%hbc_stat, \%hsingle_stat, \%hgene_map);
sub get_tags_info()
{
    my ($infile, $outfile, $ahbc, $ahgene, $ahgeneori, $ahstat, $ahbc_stat, $ahsingle_stat, $ahgene_map) = @_ ;

    my %href = ('C1'=>1, 'C2'=>1, 'C3'=>1, 'UB'=>1, 'GN'=>1);
    open (IN, "samtools view -F 4 $infile |") || die "samtools view -F 4 $infile |, $!\n" ;
    my $total_reads = 0 ;
    while(<IN>){
        chomp ;
        my ($id, $flag, $chr, $start, $mapQ, $cigar, undef, undef, undef, $seq, $qual, @tags) = split ;
        $total_reads++ ;
        my %htags = ();
        for (my $i=0; $i<@tags; $i++){
            my ($key, undef, $value) = split (/\:/, $tags[$i], 3) ;
            if (defined $href{$key}){
                next if ($value eq '' || $value eq 'NA' || $value eq 'N');
                if ($key eq 'UB'){
                    next if (length $value != 12);
                }
                if ($key eq 'C1' || $key eq 'C2' || $key eq 'C3'){
                    $value = $ahbc->{$value} ;
                }
                $htags{$key} = $value ;
            }
        }
        if (defined $htags{GN}){
            my $flag = &get_map_percent($start, $cigar, $htags{GN}, $ahgene);
            if ($flag == 0){
                $ahgene_map->{'Intronic'}++ ;
            }
            else{
                $ahgene_map->{'Exonic'}++ ;
            }
        }
        else{
            $ahgene_map->{'Intergenic'}++ ;
        }
        my @bc_types = ();
        for my $bc (qw/1 2 3/){
            if (defined $htags{"C$bc"} && $htags{"C$bc"} !~ m/N/){
                push @bc_types, "bc$bc" ;
                $ahsingle_stat->{"bc$bc"}++ ;
            }
        }
        if (scalar @bc_types != 0){
            my $bc_type = join("-", @bc_types) ;
            $ahbc_stat->{$bc_type}++ ;
        }
        if (scalar keys %htags == 5){
            my $bc = "$htags{C1}-$htags{C2}-$htags{C3}" ;
            $ahstat->{$bc}{$htags{GN}}{$htags{UB}}{$id}++ ;
        }
        else{
        }
    }
    close(IN);

    return($total_reads);
}

#my $flag = &get_map_percent($start, $cigar, $htags{GN}, $ahgene);
sub get_map_percent()
{
    my ($start, $cigar, $gene, $ahgene) = @_ ;

    return (0) if (!defined $ahgene->{$gene}) ;
    my @map_region = &get_map_region($start, $cigar);

    my @gene_region = @{$ahgene->{$gene}} ;
    my $flag = 0 ;
    for (my $i=0; $i<@map_region; $i++){
        my ($map_start, $map_end) = @{$map_region[$i]} ;
        for (my $j=0; $j<@gene_region; $j++){
            my ($start, $end) = @{$gene_region[$j]} ;
            if ($end >= $map_start){
                if ($start <= $map_end){
                    $flag = 1 ;
                }
                last ;
            }
            else{
                next ;
            }
        }
        if ($flag == 1){
            last ;
        }
    }
    return ($flag) ;
}

#my @map_region = &get_map_region($start, $cigar);
sub get_map_region()
{
    my ($ref_start, $cigar) = @_ ;

    my $len = 0 ;
    my @matchs = ();
    my $read_maplen = 0 ;
    while($cigar =~ m/(\d+)([a-zA-Z])/g){
        my ($num, $type) = ($1, $2) ;
        if ($type eq 'M' || $type eq '='){
            my $start = $len ;
            $len += $num ;
            my $end = $len ;
            push @matchs, [$start+$ref_start, $end+$ref_start-1] ;
            $read_maplen += $end - $start ;
        }
        elsif ($type eq 'N' || $type eq 'D' || $type eq 'X'){
            $len += $num ;
        }
    }

    return(@matchs);
}

#&output(\%hstat, $outfile);
sub output()
{
    my ($ahstat, $outfile) = @_ ;

    open (OUT, ">$outfile.stat") || die "$outfile.stat, $!\n" ;
    for my $bc (keys %{$ahstat}){
        for my $gene (keys %{$ahstat->{$bc}}){
            my $umi_num = scalar(keys %{$ahstat->{$bc}{$gene}});
            for my $umi (keys %{$ahstat->{$bc}{$gene}}){
                my $read_num = scalar(keys %{$ahstat->{$bc}{$gene}{$umi}});
                for my $id (keys %{$ahstat->{$bc}{$gene}{$umi}}){
                    print OUT "$bc\t$gene\t$umi_num\t$umi\t$read_num\t$id\n" ;
                }
            }
        }
    }
    close(OUT);

    return ;
}

