#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","d=s","o=s","h" );

#&help()if(defined $opts{h});
if((!defined($opts{i}) && !defined $opts{d})|| !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           tags bamfile                 <infile>     optional with -d
		-d           tags bamfiles dir            <indir>      optioanl with -i
		-o           result file                  <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $outfile = $opts{o} ;

## 
my %hstat = ();
my %hbc_stat = ();
my %hsingle_stat = ();
my $total_reads = 0 ;
open (OUT, ">$outfile.full") || die "$outfile.full, $!\n" ;
open (PAT, ">$outfile.part") || die "$outfile.part, $!\n" ;
if (defined $opts{i}){
    my $infile = $opts{i} ;
    &show_log("process $infile");
    $total_reads = &get_tags_info($infile, $outfile, \%hstat, \%hbc_stat, \%hsingle_stat);
}
elsif (defined $opts{d}){
    my $indir = $opts{d} ;
    my @files = glob("$indir/*.bam") ;
    for my $infile (@files){
        &show_log("process $infile");
        $total_reads += &get_tags_info($infile, $outfile, \%hstat, \%hbc_stat, \%hsingle_stat);
    }
}
close(OUT);
close(PAT);

open (STAT, ">$outfile.bc_stat") || die "$outfile.bc_stat, $!\n" ;
print STAT "Total_reads\t", $total_reads, "\n" ;
for my $bc (sort keys %hbc_stat){
    my $num = $hbc_stat{$bc} ;
    my $perc = int($num/$total_reads*10000+0.5)/100 ;
    print STAT "$bc\t$num\t$perc\n" ;
}
print STAT "=========== single stat ============\n" ;
for my $bc (sort keys %hsingle_stat){
    my $num = $hsingle_stat{$bc} ;
    my $perc = int($num/$total_reads*10000+0.5)/100 ;
    print STAT "$bc\t$num\t$perc\n" ;
}
close(STAT) ;

## out put
&output(\%hstat, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#$total_reads = &get_tags_info($infile, $outfile, \%hstat, \%hbc_stat, \%hsingle_stat);
sub get_tags_info()
{
    my ($infile, $outfile, $ahstat, $ahbc_stat, $ahsingle_stat) = @_ ;

    my %href = ('C1'=>1, 'C2'=>1, 'C3'=>1, 'UB'=>1, 'GN'=>1);
    open (IN, "samtools view -F 4 $infile |") || die "samtools view -F 4 $infile |, $!\n" ;
    my $total_reads = 0 ;
    while(<IN>){
        chomp ;
        my ($id, $flag, $chr, $start, $mapQ, $cigar, undef, undef, undef, $seq, $qual, @tags) = split ;
        $total_reads++ ;
        my %htags = ();
        for (my $i=0; $i<@tags; $i++){
            my ($key, undef, $value) = split /\:/, $tags[$i] ;
            if (defined $href{$key}){
                next if ($value eq '' || $value eq 'NA' || $value eq 'N');
                if ($key eq 'UB'){
                    next if (length $value != 12);
                }
                $htags{$key} = $value ;
            }
        }
        my @bc_types = ();
        for my $bc (qw/1 2 3/){
            if (defined $htags{"C$bc"} && $htags{"C$bc"} !~ m/N/){
                push @bc_types, "bc$bc" ;
                $ahsingle_stat->{"bc$bc"}++ ;
            }
        }
        if (scalar @bc_types != 0){
            my $bc_type = join("-", @bc_types) ;
            $ahbc_stat->{$bc_type}++ ;
        }
        if (scalar keys %htags == 5){
            my $bc = "$htags{C1}-$htags{C2}-$htags{C3}" ;
            print OUT "$id\t$bc\t$htags{GN}\t$htags{UB}\n" ;
            $ahstat->{$bc}{$htags{GN}}{$htags{UB}}{$id}++ ;
        }
        else{
            print PAT "$id" ;
            for my $key (sort keys %htags){
                print PAT "\t$key:$htags{$key}" ;
            }
            print PAT "\n" ;
        }
    }
    close(IN);


    return($total_reads);
}

#&output(\%hstat, $outfile);
sub output()
{
    my ($ahstat, $outfile) = @_ ;

    open (OUT, ">$outfile.stat") || die "$outfile.stat, $!\n" ;
    for my $bc (keys %{$ahstat}){
        for my $gene (keys %{$ahstat->{$bc}}){
            my $umi_num = scalar(keys %{$ahstat->{$bc}{$gene}});
            for my $umi (keys %{$ahstat->{$bc}{$gene}}){
                my $read_num = scalar(keys %{$ahstat->{$bc}{$gene}{$umi}});
                for my $id (keys %{$ahstat->{$bc}{$gene}{$umi}}){
                    print OUT "$bc\t$gene\t$umi_num\t$umi\t$read_num\t$id\n" ;
                }
            }
        }
    }
    close(OUT);

    return ;
}

