#!/usr/bin/perl -w
# 
# Copyright (c) BIO_MARK 2021
# Writer:         Dengdj <dengdj@biomarker.com.cn>
# Program Date:   2021
# Modifier:       Dengdj <dengdj@biomarker.com.cn>
# Last Modified:  2021
my $ver="1.0";

use strict;
use Getopt::Long;
use Data::Dumper;
use FindBin qw($Bin $Script);
use File::Basename qw(basename dirname);

#####################

my %opts;
GetOptions(\%opts,"i=s","d=s","b=s","g=s","o=s","h" );

#&help()if(defined $opts{h});
if((!defined($opts{i}) && !defined $opts{d})|| !defined($opts{g}) || !defined($opts{b}) || !defined($opts{o}) || defined($opts{h}))
{
	print <<"	Usage End.";
	Description:
		
		Version: $ver

	Usage:

		-i           tags bamfile                 <infile>     optional with -d
		-d           tags bamfiles dir            <indir>      optioanl with -i
		-b           barcode seq file             <infile>     must be given
		-g           gtf file                     <infile>     must be given
		-o           result file                  <outfile>    must be given
		-h           Help document

	Usage End.

	exit;
}

###############Time
my $Time_Start;
$Time_Start = sub_format_datetime(localtime(time()));
&show_log("Start Time :[$Time_Start]");
################
$| = 1 ;
## get parameters
my $outfile = $opts{o} ;
my $bcfile = $opts{b} ;
my $gtffile = $opts{g} ;

## read barcode file
my %hbc = ();
&get_barcode($bcfile, \%hbc);

## get gene info
my %hgene = ();
my %hgeneori = ();
my %hgenelen = ();
&get_gene_info($gtffile, \%hgene, \%hgenelen, \%hgeneori);

## 
my %hstat = ();
my %hbc_stat = ();
my %hsingle_stat = ();
my $total_reads = 0 ;
open (OUT, ">$outfile.full") || die "$outfile.full, $!\n" ;
if (defined $opts{i}){
    my $infile = $opts{i} ;
    &show_log("process $infile");
    $total_reads = &get_tags_info($infile, $outfile, \%hbc, \%hgene, \%hgenelen, \%hgeneori, \%hstat, \%hbc_stat, \%hsingle_stat);
}
elsif (defined $opts{d}){
    my $indir = $opts{d} ;
    my @files = glob("$indir/*.bam") ;
    for my $infile (@files){
        &show_log("process $infile");
        $total_reads += &get_tags_info($infile, $outfile, \%hbc, \%hgene, \%hgenelen, \%hgeneori, \%hstat, \%hbc_stat, \%hsingle_stat);
    }
}
close(OUT);

open (STAT, ">$outfile.bc_stat") || die "$outfile.bc_stat, $!\n" ;
print STAT "Total_reads\t", $total_reads, "\n" ;
for my $bc (sort keys %hbc_stat){
    my $num = $hbc_stat{$bc} ;
    my $perc = int($num/$total_reads*10000+0.5)/100 ;
    print STAT "$bc\t$num\t$perc\n" ;
}
print STAT "=========== single stat ============\n" ;
for my $bc (sort keys %hsingle_stat){
    my $num = $hsingle_stat{$bc} ;
    my $perc = int($num/$total_reads*10000+0.5)/100 ;
    print STAT "$bc\t$num\t$perc\n" ;
}
close(STAT) ;

## out put
&output(\%hstat, $outfile);

###############Time
my $Time_End;
$Time_End = sub_format_datetime(localtime(time()));
&show_log("End Time :[$Time_End]");

###############Subs
sub sub_format_datetime {#Time calculation subroutine
    my($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = @_;
	$wday = $yday = $isdst = 0;
    sprintf("%4d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $day, $hour, $min, $sec);
}

sub ABSOLUTE_DIR
{ #$pavfile=&ABSOLUTE_DIR($pavfile);
	my $cur_dir=`pwd`;
	$cur_dir =~ s/\n$//;
	my ($in)=@_;
	my $return="";
	
	if(-f $in)
	{
		my $dir=dirname($in);
		my $file=basename($in);
		chdir $dir;$dir=`pwd`;
		$dir =~ s/\n$// ;
		$return="$dir/$file";
	}
	elsif(-d $in)
	{
		chdir $in;$return=`pwd`;
		$return =~ s/\n$// ;
	}
	else
	{
		warn "Warning just for file and dir [$in]\n";
		exit;
	}
	
	chdir $cur_dir;
	return $return;
}

# &show_log("txt")
sub show_log()
{
	my ($txt) = @_ ;
	my $time = time();
	my $Time = &sub_format_datetime(localtime($time));
	print "$Time:\t$txt\n" ;
	return ($time) ;
}

#&run_or_die($cmd);
sub run_or_die()
{
	my ($cmd) = @_ ;
	my $start_time = &show_log($cmd);
	my $flag = system($cmd) ;
	if ($flag != 0){
		my $end_time = &show_log("Error: command fail: $cmd");
		&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
		exit(1);
	}
	my $end_time = &show_log("done.");
	&show_log("Elaseped time: ".($end_time - $start_time)."s\n");
	return ;
}

## qsub
sub qsub()
{
	my ($shfile, $maxproc) = @_ ;
	my $dir = dirname($shfile) ;
	my $cmd = "cd $dir && sh /share/nas2/genome/bmksoft/tool/qsub_sge_plus/v1.0/qsub_sge.plus.sh --maxproc $maxproc --reqsub --independent $shfile" ;
	&run_or_die($cmd);

	return ;
}

## qsub_mult($shfile, $max_proc, $job_num)
sub qsub_mult()
{
	my ($shfile, $max_proc, $job_num) = @_ ;
	if ($job_num > 500){
		my @shfiles = &cut_shfile($shfile);
		for my $file (@shfiles){
			&qsub($file, $max_proc);
		}
	}
	else{
		&qsub($shfile, $max_proc) ;
	}
}

#my @shfiles = &cut_shfile($shfile);
sub cut_shfile()
{
	my ($file) = @_ ;
	my @files = ();
	my $num = 0 ;
	open (IN, $file) || die "Can't open $file, $!\n" ;
	(my $outprefix = $file) =~ s/.sh$// ;
	while (<IN>){
		chomp ;
		if ($num % 500 == 0){
			close(OUT);
			my $outfile = "$outprefix.sub_".(int($num/500)+1).".sh" ;
			push @files, $outfile ;
			open (OUT, ">$outfile") || die "Can't creat $outfile, $!\n" ;
		}
		print OUT $_, "\n" ;
		$num ++ ;
	}
	close(IN);
	close(OUT);

	return @files ;
}

sub sub_normal_dis_ran(){#$average,$standard_deviation
        my ($aver,$stand_dev) = @_ ;
        my $ran1 = rand() ;
        my $ran2 = rand() ;
        my $y = ((-2*log($ran1))**(0.5))*sin(2*3.1415926535*$ran2) ;
        return ($aver + $stand_dev*$y) ;
}

#&get_barcode($bcfile, \%hbc);
sub get_barcode()
{
    my ($bcfile, $ahbc) = @_ ;

    open (IN, $bcfile) || die "$bcfile, $!\n" ;
    $/ = ">" ;
    while(<IN>){
        chomp ;
        next if (m/^\s*$/);
        my ($id, $seq) = split(/\n/, $_, 2) ;
        $id = (split /\s+/, $id)[0] ;
        $seq =~ s/\n//g ;
        $ahbc->{$seq} = $id ;
    }
    $/ = "\n" ;
    close(IN);

    return ;
}

#&get_gene_info($gtffile, \%hgene, \%hgenelen, \%hgeneori);
sub get_gene_info()
{
    my ($gtffile, $ahgene, $ahgenelen, $ahgeneori) = @_ ;

    open (IN, $gtffile) || die "$gtffile, $!\n" ;
    my %htrans = ();
    while(<IN>){
        chomp ;
        next if (m/^\#|^\s*$/);
        my ($chr, $soft, $type, $start, $end, undef, $ori, $offset, $info) = split(/\t+/, $_, 9) ;
        next if ($type ne 'exon') ;
        my @infos = split /\s+/, $info ;
        my %hinfos = ();
        for (my $i=0; $i<@infos; $i+=2){
            $hinfos{$infos[$i]} = $infos[$i+1] ;
        }
        my $gene_id = $hinfos{"gene_id"} ;
        $gene_id =~ s/[\"\;]//g ;
        my $trans_id = $hinfos{"transcript_id"} ;
        $trans_id =~ s/[\"\;]//g ;
        push @{$htrans{$gene_id}{$trans_id}}, [$start, $end] ;
        $ahgeneori->{$gene_id} = $ori ;
    }
    close(IN);

    for my $gene (keys %htrans){
        for my $trans (keys %{$htrans{$gene}}){
            my $len = 0 ;
            for (my $i=0; $i<@{$htrans{$gene}{$trans}}; $i++){
                $len += $htrans{$gene}{$trans}[$i][1] - $htrans{$gene}{$trans}[$i][0] + 1 ;
            }
            $ahgenelen->{$gene}{$trans} = $len ;
            @{$ahgene->{$gene}{$trans}} = sort {$a->[0]<=>$b->[0] || $a->[1]<=>$b->[1]} @{$htrans{$gene}{$trans}} ;
        }
    }

    return ;
}

#$total_reads = &get_tags_info($infile, $outfile, \%hbc, \%hgene, \%hgenelen, \%hgeneori, \%hstat, \%hbc_stat, \%hsingle_stat);
sub get_tags_info()
{
    my ($infile, $outfile, $ahbc, $ahgene, $ahgenelen, $ahgeneori, $ahstat, $ahbc_stat, $ahsingle_stat) = @_ ;

    my %href = ('C1'=>1, 'C2'=>1, 'C3'=>1, 'UB'=>1, 'GN'=>1);
    open (IN, "samtools view -F 4 $infile |") || die "samtools view -F 4 $infile |, $!\n" ;
    my $total_reads = 0 ;
    while(<IN>){
        chomp ;
        my ($id, $flag, $chr, $start, $mapQ, $cigar, undef, undef, undef, $seq, $qual, @tags) = split ;
        $total_reads++ ;
        my %htags = ();
        for (my $i=0; $i<@tags; $i++){
            my ($key, undef, $value) = split /\:/, $tags[$i] ;
            if (defined $href{$key}){
                next if ($value eq '' || $value eq 'NA' || $value eq 'N');
                if ($key eq 'UB'){
                    next if (length $value != 12);
                }
                if ($key eq 'C1' || $key eq 'C2' || $key eq 'C3'){
                    $value = $ahbc->{$value} ;
                }
                $htags{$key} = $value ;
            }
        }
        my @bc_types = ();
        for my $bc (qw/1 2 3/){
            if (defined $htags{"C$bc"} && $htags{"C$bc"} !~ m/N/){
                push @bc_types, "bc$bc" ;
                $ahsingle_stat->{"bc$bc"}++ ;
            }
        }
        if (scalar @bc_types != 0){
            my $bc_type = join("-", @bc_types) ;
            $ahbc_stat->{$bc_type}++ ;
        }
        if (scalar keys %htags == 5){
            my $bc = "$htags{C1}-$htags{C2}-$htags{C3}" ;
            my ($map_gene_start_perc, $map_gene_end_perc, $map_start, $map_end) = &get_map_percent($start, $cigar, $htags{GN}, $ahgene, $ahgenelen);
            if ($ahgeneori->{$htags{GN}} eq '-'){
                my $start = 100 - $map_gene_end_perc ;
                my $end = 100 - $map_gene_start_perc ;
                ($map_gene_start_perc, $map_gene_end_perc) = ($start, $end) ;
            }
            if ($map_gene_end_perc == $map_gene_end_perc){
                ($map_gene_start_perc, $map_gene_end_perc) = (-1, -1) ;
            }
            print OUT "$id\t$bc\t$htags{GN}\t$htags{UB}\t$map_gene_start_perc\t$map_gene_end_perc\t$map_start\t$map_end\n" ;
            $ahstat->{$bc}{$htags{GN}}{$htags{UB}}{$id}++ ;
        }
        else{
        }
    }
    close(IN);

    return($total_reads);
}

#my ($map_gene_start, $map_gene_end) = &get_map_percent($start, $cigar, $htags{GN}, $ahgene, $ahgenelen);
sub get_map_percent()
{
    my ($start, $cigar, $gene, $ahgene, $ahgenelen) = @_ ;

    my @map_region = &get_map_region($start, $cigar);
    my $map_start = $map_region[0][0] ;
    my $map_end = $map_region[-1][1] ;

    my ($max_start_percent, $max_end_percent) = (0, 0) ;
    my $max_region = 0 ;
    for my $trans (keys %{$ahgene->{$gene}}){
        my $map_start_len = 0 ;
        my $map_end_len = 0 ;
        my $genelen = $ahgenelen->{$gene}{$trans} ;
        my @gene_region = @{$ahgene->{$gene}{$trans}} ;
        for (my $i=0; $i<@gene_region; $i++){
            my ($start, $end) = @{$gene_region[$i]} ;
            if ($end >= $map_start){
                if ($start <= $map_start){
                    $map_start_len += $map_start - $start + 1 ;
                }
                else{
                }
                last ;
            }
            $map_start_len += $end - $start + 1 ;
        }
        for (my $i=0; $i<@gene_region; $i++){
            my ($start, $end) = @{$gene_region[$i]} ;
            if ($end >= $map_end){
                if ($start <= $map_end){
                    $map_end_len += $map_end - $start + 1 ;
                }
                last ;
            }
            $map_end_len += $end - $start + 1 ;
        }
        my $start_percent = int($map_start_len/$genelen*100+0.5) ;
        my $end_percent = int($map_end_len/$genelen*100+0.5) ;
        if ($end_percent - $start_percent + 1 > $max_region){
            $max_region = $end_percent - $start_percent + 1 ;
            $max_start_percent = $start_percent ;
            $max_end_percent = $end_percent ;
        }
    }

    return($max_start_percent, $max_end_percent, $map_start, $map_end);
}

#my @map_region = &get_map_region($start, $cigar);
sub get_map_region()
{
    my ($ref_start, $cigar) = @_ ;

    my $len = 0 ;
    my @matchs = ();
    my $read_maplen = 0 ;
    while($cigar =~ m/(\d+)([a-zA-Z])/g){
        my ($num, $type) = ($1, $2) ;
        if ($type eq 'M' || $type eq '='){
            my $start = $len ;
            $len += $num ;
            my $end = $len ;
            push @matchs, [$start+$ref_start, $end+$ref_start-1] ;
            $read_maplen += $end - $start ;
        }
        elsif ($type eq 'N' || $type eq 'D' || $type eq 'X'){
            $len += $num ;
        }
    }

    return(@matchs);
}

#&output(\%hstat, $outfile);
sub output()
{
    my ($ahstat, $outfile) = @_ ;

    open (OUT, ">$outfile.stat") || die "$outfile.stat, $!\n" ;
    for my $bc (keys %{$ahstat}){
        for my $gene (keys %{$ahstat->{$bc}}){
            my $umi_num = scalar(keys %{$ahstat->{$bc}{$gene}});
            for my $umi (keys %{$ahstat->{$bc}{$gene}}){
                my $read_num = scalar(keys %{$ahstat->{$bc}{$gene}{$umi}});
                for my $id (keys %{$ahstat->{$bc}{$gene}{$umi}}){
                    print OUT "$bc\t$gene\t$umi_num\t$umi\t$read_num\t$id\n" ;
                }
            }
        }
    }
    close(OUT);

    return ;
}

